
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : configuration.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.6 $
**
**  DESCRIPTION      :  Implementation of configuration class
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Jul 23, 2002
**      LAST UPDATE     : Apr 13, 2007
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: configuration.cpp,v $
   Revision 1.6  2007/05/04 14:01:22  arjen
   Added new arguments to configuration::find_parameter() to allow for a list of
   sections and a list of parameters in sections.
   The new arguments are section index and parameter index (default = 0).

   Revision 1.5  2003/03/29 07:19:37  arjen
   Bugfix: Do not read the config file in the home
   directory if the HOME environment variable doesn't exist

   Revision 1.4  2002/11/23 12:48:18  arjen
   Check if a configuration file exists before feeding it to the XML parser.

   Revision 1.3  2002/11/02 14:35:15  arjen
   Uses the XML2 library
   BugFix: segfault when finding a parameters that is not in the user-sepcific
   config tree.

   Revision 1.2  2002/09/28 06:48:46  arjen
   Bugfix: In configuration::find_parameter(), check if the parameter node exists
   before retrieving its contents.

   Revision 1.1  2002/07/25 08:01:26  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: configuration.cpp,v 1.6 2007/05/04 14:01:22 arjen Exp $";

#include <fcntl.h>
#include <unistd.h>
#include "configuration.h"


/*=========================================================================
**  NAME           : configuration::read
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : true if any config files were found
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Nov 22, 2002
**=========================================================================
*/

bool configuration::read(const String name)
{
   String     filename;

   app_name   = name;

   //   Try to read the config files.
 
   filename = "/etc/" + name + ".conf";
   //  Check if we can read the file.

   int fd = open(filename, O_RDONLY);
   if (fd < 0)
   {
      filename = "/usr/local/etc/" + name + ".conf";
      fd = open(filename, O_RDONLY);
   }

   if (fd > 0)
   {
      close(fd);
      system = xmlParseFile(filename);
   }

   if (getenv("HOME") != NULL)
   {
      filename = getenv("HOME");
      filename += "/.";
      filename += name + ".conf";
      fd = open(filename, O_RDONLY);
      if (fd > 0)
      {
         close(fd);
         user = xmlParseFile(filename);
      }
   }

   //  Check the root element, which must be the application name

   xmlNodePtr   root;

   if (system != NULL)
   {
      root = xmlDocGetRootElement(system);
      if (app_name != (const char *)root->name)
      {
         std::cerr << "Wrong config file.\n";
         xmlFreeDoc(system);
         system = NULL;
      }
   }

   if (user != NULL)
   {
      root = xmlDocGetRootElement(user);
      if (app_name != (const char *)root->name)
      {
         std::cerr << "Wrong config file.\n";
         xmlFreeDoc(user);
         user = NULL;
      }
   }

   return system != NULL || user != NULL;
}

/*=========================================================================
**  NAME           : configuration::xmlFindTag
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : Pointer to the XML element or NULL
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Apr 13, 2007
**=========================================================================
*/

xmlNodePtr configuration::xmlFindTag(xmlNodePtr node, const String tag, int n)
{
   xmlNodePtr   element = NULL;

   while (element == NULL && node != NULL)
   {
      if (node->type == XML_ELEMENT_NODE && tag == (char *)node->name)
      {
         if (n == 0)
         {
            element = node;
         }
         n--;
      }
      node = node->next;
   }

   return element;
}

/*=========================================================================
**  NAME           : configuration::find_parameter
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : The content of the parameter or an empty string
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Apr 13, 2007
**=========================================================================
*/

String configuration::find_parameter(const String section, const String parameter, int sec_n, int param_n)
{
   xmlNodePtr     root_node;
   xmlNodePtr     section_node;
   xmlNodePtr     param_node;

   String         param_value("");

   //  First, we try to find the parameter in the system-wide config

   if (system)
   {
      root_node    = xmlDocGetRootElement(system);
      section_node = xmlFindTag(root_node->children, section, sec_n);
      if (section_node != NULL)
      {
         param_node = xmlFindTag(section_node->children, parameter, param_n);
         if (param_node != NULL)
         {
            param_node = param_node->children;
         }
         if (param_node != NULL)
         {
            param_value = (char *)param_node->content;
         }
      }
   }

   //  If the parameter is also defined in the user config, it will override
   //  the system-wide value.

   if (user)
   {
      root_node    = xmlDocGetRootElement(user);
      section_node = xmlFindTag(root_node->children, section, sec_n);
      if (section_node != NULL)
      {
         param_node = xmlFindTag(section_node->children, parameter, param_n);
         if (param_node != NULL)
         {
            param_node = param_node->children;
         }
         if (param_node != NULL)
         {
            param_value = (char *)param_node->content;
         }
      }
   }

   return param_value;
}
