
/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : edit.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Implementation of text editor classes
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 13, 1998
**      LAST UPDATE     : Jun 18, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: edit.cpp,v $
   Revision 1.1  2002/07/25 08:01:26  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: edit.cpp,v 1.1 2002/07/25 08:01:26 arjen Exp $";

#include "edit.h"
#include <X11/keysym.h>

#include <stdio.h>

/*=========================================================================
**  NAME           : EV_KeyPress
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : May 21, 1998
**=========================================================================
*/

int edit::EV_KeyPress(XKeyEvent ev)
{
   KeySym   key;
   int      x_cursor;
   unsigned w, h;

   Size(w, h);   //  The size of the window

   toggle_cursor();
   key = XLookupKeysym(&ev, ev.state & 1);

   //  If text is selected, remove it first.

   DeleteSelection();

   switch (key)
   {
   case XK_Left:
      if (cursor_position > 0)
      {
         cursor_position--;
      }
      break;

   case XK_Right:
      if (cursor_position < ~text)
      {
         cursor_position++;
      }
      break;

   case XK_BackSpace:
      if (cursor_position > 0)
      {
         cursor_position--;
         text(cursor_position, 1) = "";
      }
      break;

   case XK_Delete:
      text(cursor_position, 1) = "";
      break;

   case XK_Return:
   case XK_KP_Enter:
      enter();
      break;

   default:
      if (key >= 0x20 && key <= 0x7e)
      {
         text(cursor_position, 0) = String((char)key);
         cursor_position++;
      }
      break;

   }
   if (cursor_position != 0)
      x_cursor = fixed.TextWidth(text,cursor_position) + text_position;
   else
      x_cursor = text_position;

   if (x_cursor < 0)
   {
      text_position += w/4;
   }
   else if ((unsigned)x_cursor > w)
   {
      text_position -= w / 4;
   }
   redraw();   //  Recreates the cursor.

   return 1;
}

/*=========================================================================
**  NAME           : EV_ButtonPress - Handle press of mouse button
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : Buttonpress of the left mouse button puts the text
**                   cursor at the mouse's position and sets the focus to
**                   the edit window.
**                   The X server will send a FocusIn event as a result of
**                   the SetFocus
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : May 22, 1998
**=========================================================================
*/


int edit::EV_ButtonPress(XButtonEvent ev)
{
   if (selected_until != -1)
   {
      selected_until = -1;
      redraw();
   }

   if (ev.button == 1)
   { 
      //  Left mouse button

      if (focus_state != No_Focus)
      {
         //  The cursor is visible; hide it from its old position

         toggle_cursor();
      }
      cursor_position = map_to_textposition(ev.x);
      if (cursor_position > ~text)
         cursor_position = ~text;

      SetFocus();

      if (focus_state != No_Focus)
      {
         //  If we already have the focus, show the cursor on its new
         //  position, else the FocusIn event will take care of this.

         toggle_cursor();
      }
   }
   else if (ev.button == 2)
   {
      // Middle mouse button, paste the cut buffer

      int   nbytes;
      char  *paste_text;

      paste_text = XFetchBytes(stddpy, &nbytes);
      if (paste_text)
      {
         printf(" Pasting %s\n", paste_text);
         text(cursor_position, 0) = paste_text;
         redraw();
      }
   }
   return 1;
}

/*=========================================================================
**  NAME           : EV_FocusIn - Handle FocusIn event
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : Four event handling functions work together to track
**                   the keyboard focus. The events involved are FocusIn,
**                   FocusOut, EnterNotify and LeaveNotify
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : May 21, 1998
**=========================================================================
*/

int edit::EV_FocusIn(XFocusChangeEvent ev)
{
   if (focus_state == No_Focus)
      toggle_cursor();
   focus_state = Explicit_Focus;
   return 1;
}

int edit::EV_FocusOut(XFocusChangeEvent ev)
{
   focus_state = No_Focus;
   toggle_cursor();
   return 1;
}
int edit::EV_EnterNotify(XCrossingEvent ev)
{
   if (ev.focus && focus_state == No_Focus)
   {
      focus_state = Implicit_Focus;
      toggle_cursor();
   }
   return 1;
}

int edit::EV_LeaveNotify(XCrossingEvent ev)
{
   if (ev.focus && focus_state == Implicit_Focus)
   {
      focus_state = No_Focus;
      toggle_cursor();
   }
   return 1;
}


/*=========================================================================
**  NAME           : EV_MotionNotify - handle motion event.
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : May 22, 1998
**=========================================================================
*/

int edit::EV_MotionNotify(XMotionEvent ev)
{
   selected_until = map_to_textposition(ev.x);

   redraw();

   return 1;
}

int edit::EV_ButtonRelease(XButtonEvent ev)
{
   String selected_text;
   int    selection_length;

   if (ev.button == 1 && selected_until != -1)
   {
      selected_until = map_to_textposition(ev.x);

      if (selected_until >= cursor_position)
      {
         selection_length = selected_until - cursor_position;
         selected_text = text(cursor_position, selection_length);
      }
      else
      {
         selection_length = cursor_position - selected_until;
         selected_text = text(selected_until, selection_length);
      }
      if (selected_text)
      {
         printf("%d chars selected: %s\n", selection_length,(char *)selected_text);

         XStoreBytes(stddpy, selected_text, selection_length);
      }
      else
      {
         printf("Nothing selected\n");
      }
   }

   return 1;
}

int edit::map_to_textposition(int x)
{

   return (x - text_position) / 6;
}

int edit::map_to_pixelposition(int tp)
{
      return fixed.TextWidth(text, tp) + text_position;
}

/*=========================================================================
**  NAME           : toggle_cursor
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 17, 1998
**=========================================================================
*/

void edit::toggle_cursor(void)
{
   int  x_cursor;

   if (cursor_position != 0)
      x_cursor = fixed.TextWidth(text,cursor_position) + text_position;
   else
      x_cursor = 0;

   DrawLine(edit_cursor_gc, x_cursor, 1, x_cursor, 13);
}

/*=========================================================================
**  NAME           : redraw - Draw the edit object (again)
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : Clear the window and redraw the edited text.
**                   If part of the text is selected, draw this part
**                   in a different color.
**                   Draw a text cursor if we have the focus.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : May 22, 1998
**=========================================================================
*/

void edit::redraw(void)
{
   int    selection_length;
   String selected_text;
   int    x_start, x_end;

   /*  Redraw the entire text */

   Clear();
   DrawString(text_normal_gc, text_position, 12, text);

   /*  If part of the text is selected, highlight this part */

   if (selected_until != -1)
   {
      if (selected_until >= cursor_position)
      {
         selection_length = selected_until - cursor_position;
         x_start = map_to_pixelposition(cursor_position);
         x_end   = map_to_pixelposition(selected_until);
         selected_text = text(cursor_position, selection_length);
      }
      else
      {
         selection_length = cursor_position - selected_until;
         x_start = map_to_pixelposition(selected_until);
         x_end   = map_to_pixelposition(cursor_position);
         selected_text = text(selected_until, selection_length);
      }

      XFillRectangle(stddpy, ID(), inside_3D_gc, x_start, 1, x_end-x_start, 14);
      DrawString(text_normal_gc, x_start, 12, selected_text);
   }

   /*  If we have the focus, draw the text cursor */

   if (focus_state != No_Focus)
   {
      toggle_cursor();
   }
}

void edit::enter(void)
{
   ParentMessage(EDIT_ENTERED);
}

void edit::focuslost(void)
{
   ParentMessage(EDIT_FOCUSLOST);
}


/*=========================================================================
**  NAME           : DeleteSelection - Delete any selected text
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : The number of characters deleted.
**
**  DESCRIPTION    : 
**                   Make sure the cursor stays within the text. If the
**                   selection is 'right-to-left', i.e. from selected_until
**                   to cursor_position, the cursor is moved to the beginning
**                   of the selected text.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Jun 18, 1998
**=========================================================================
*/

int edit::DeleteSelection(void)
{
   int  selection_length;

   selection_length = 0;

   if (selected_until != -1)
   {

      if (selected_until >= cursor_position)
      {
         selection_length = selected_until - cursor_position;
         text(cursor_position, selection_length) = "";
      }
      else
      {
         selection_length = cursor_position - selected_until;
         text(selected_until, selection_length) = "";
         cursor_position = selected_until;
      }
   }
   selected_until = -1;  // Nothing selected

   return selection_length;
}

/*=========================================================================
**  NAME           : ClearSelection
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Jun 18, 1998
**=========================================================================
*/

int edit::ClearSelection(void)
{
   if (selected_until != -1)
   {
      selected_until = -1;
      redraw();
   }

   return 0;
}

/*=========================================================================
**  NAME           : SelectAll
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Jun 18, 1998
**=========================================================================
*/

void edit::SelectAll(void)
{
   cursor_position = 0;
   selected_until = ~text - 1;
   redraw();

}
