/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : menu.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      : Definition of menu classes 
**
**  EXPORTED OBJECTS : class menu_container
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 13, 1998
**      LAST UPDATE     : Mar 07, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: menu.h,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: menu.h,v 1.1 2002/07/25 08:01:27 arjen Exp $"; */

#include <X11/Xlib.h>
#include "xappl.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : menu_container
//  BASECLASS      : window
//  MEMBERS        : mode : Horizontal or Vertical
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    : A menu_container is the parent window of menu items.
//                   The menu_container lines the menu items up next to
//                   eachother and makes them all fit together.
//
//  RELATIONS      : shown_by : menu_item that popped me up. The menu item
//                              which is clicked uses this to climb to
//                              the top of the menu tree and hide all submenus
//  SEE ALSO       :
//  LAST MODIFIED  : Feb 27, 1998
///////////////////////////////////////////////////////////////////////////
*/

class menu_item;  // Forward declaration.

class menu_container : public window
{

protected:

   enum {Horizontal, Vertical} mode;

public:

   menu_item *shown_by;

   /*
    *   Both constructors perform the same function. Create a window
    *   with default size, set the background color and solicit
    *   StructureNotify events.
    *   Note that the size of the menu_container is eventually determined
    *   by the sizes of the menu items.
    */

   menu_container() : window(0, 0, 50, 100, 1)
   {
      shown_by = 0;

      Background(menu_normal);
      SelectInput(StructureNotifyMask, 1);
   }

   menu_container(window &frame) : window(frame, 0, 0, 100, 15)
   {
      shown_by = 0;

      Background(menu_normal);
      SelectInput(StructureNotifyMask, 1);
   }

   void RearrangeItems(unsigned &overall_width, unsigned &overall_height);

   /*
    *   The position of a sub-menu depends on the mode. Horizontal menus
    *   put submenus below the item, vertical menus put a submenu right
    *   of the item.
    */

   void PositionSubmenu(int &x, int &y, unsigned w, unsigned h)
   {
      if (mode == Horizontal)
         y += h - 3;
      else
         x += w - 5;
   }

   /*
    *  Submenu management.
    */

   void HideSubmenus(void);

   void Show(menu_item *by, int x, int y);

};


/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : menu_item
//  BASECLASS      : window
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  :
///////////////////////////////////////////////////////////////////////////
*/

class menu_item : public window
{
   int   command_code;

protected:
 
   int   highlighted;
   int   enabled;

   menu_container   *popup;

public:

   menu_item(window &bar, int cc, int en = 1) : window(bar, 0, 0, 50, 15, 0)
   {
      highlighted = 0;
      enabled     = en;
      command_code = cc;
      popup        = 0;

      SelectInput(ExposureMask, 1);
      Background(menu_normal);
      SelectInput(ButtonReleaseMask|EnterWindowMask|LeaveWindowMask, enabled);
   }

   virtual void redraw(void);

   virtual int EV_Expose(XExposeEvent ev)
   {
      redraw();
      return 1;
   }

   virtual int EV_EnterNotify(XCrossingEvent ev);
   virtual int EV_LeaveNotify(XCrossingEvent ev);
   virtual int EV_ButtonRelease(XButtonEvent ev);

   void HideSubmenu(void);
   void ShowSubmenu(int x, int y);

   void Popup(menu_container *pop)
   {
      popup = pop;
   }

   void Enable(int e)
   {
      enabled = e;
      SelectInput(EnterWindowMask, e);
   }

};

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : menu_bar
//  BASECLASS      : menu_container
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  :
///////////////////////////////////////////////////////////////////////////
*/

class menu_bar : public menu_container
{
public:

   menu_bar(window &frame) : menu_container(frame)
   {
      mode = Horizontal;
   }


   virtual int ParentResized(int w, int h)
   {
      unsigned width, height;

      RearrangeItems(width, height);

      //  Resize to fit the parent window

      Resize(w-2, height);

      return 1;
   }
};


/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : popup_menu
//  BASECLASS      : menu_container
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  :
///////////////////////////////////////////////////////////////////////////
*/

class popup_menu: public menu_container
{
public:

   popup_menu() : menu_container()
   {
      XSetWindowAttributes  attrib;
      unsigned long         mask;

      attrib.override_redirect = True;
      mask                     = CWOverrideRedirect;

      XChangeWindowAttributes(stddpy.Dpy(), ID(), mask, &attrib);
      mode = Vertical;
   }

   virtual int EV_ConfigureNotify(XConfigureEvent ev)
   {
      unsigned width, height;

      RearrangeItems(width, height);

      Resize(width, height);

      return 1;
   }

};


/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : menu_label
//  BASECLASS      : menu_item
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Mar 07, 1998
///////////////////////////////////////////////////////////////////////////
*/

class menu_label : public menu_item
{
   char  *label;

public:

   menu_label(window &bar, char *text, int cc = 0) : menu_item(bar, cc)
   {
      label = text;
      int width = default_font.TextWidth(label) + 10;
      int height = default_font.ascent() + default_font.descent() + 6;
      Resize(width, height);
   }

   virtual void redraw(void);
};

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : menu_separator
//  BASECLASS      : menu_item
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  :
///////////////////////////////////////////////////////////////////////////
*/

class menu_separator : public menu_item
{
public:

   menu_separator(window &bar) : menu_item(bar, 0, 0)
   {
      Resize(2, 7);
   }

   virtual void redraw(void);
};

