/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : xappl.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.2 $
**
**  DESCRIPTION      : Definition of xapplication class 
**
**  EXPORTED OBJECTS : class xapplication
**  LOCAL    OBJECTS : class xw_list, class xw_hash
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Feb 23, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: xappl.h,v $
   Revision 1.2  2007/05/04 14:01:22  arjen
   Added new arguments to configuration::find_parameter() to allow for a list of
   sections and a list of parameters in sections.
   The new arguments are section index and parameter index (default = 0).

   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: xappl.h,v 1.2 2007/05/04 14:01:22 arjen Exp $"; */

#ifndef XAPPL_H
#define XAPPL_H

#include <X11/Xlib.h>
#include "display.h"
#include "xwindow.h"
#include "cursor.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : xw_list
//  BASECLASS      :
//  MEMBERS        : id   : X id of the window
//                   win  : Pointer to the window object
//                   next : Pointer to next xw_list
//  OPERATORS      :
//  METHODS        : xw_list(XID, window *, xw_list *)
//                      Insert at the start of the list.
//
//  DESCRIPTION    : Linked list of X-Window ID and window pointers.
//                   Is used in the hash table of windows.
//
//  RELATIONS      :
//  SEE ALSO       : xw_hash
//  LAST MODIFIED  : Feb 07, 1998
///////////////////////////////////////////////////////////////////////////
*/

class xw_list
{
public:
   XID      id;
   window   *win;
   xw_list  *next;

   xw_list(XID win_id, window *win_ptr, xw_list *list)
   {
      id   = win_id;
      win  = win_ptr;
      next = list;
   }
};


/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : xw_hash
//  BASECLASS      :
//  MEMBERS        : tabsize  : Size of the hash table.
//                   table    : Array of xw_list entries.
//  OPERATORS      : window * [XID]
//  METHODS        : xw_hash(unsigned size)
//                      Create a hash table of specific size.
//                   xh_add(XID, window *)
//                      Add a new entry to the hash table.
//                   xh_del(XID)
//                      Delete an entry from the hash table.
//
//  DESCRIPTION    : The hash table associates a window's XID with a pointer
//                   to the window object. The xapplication object uses the
//                   hash table for dispatching events to windows.
//
//                   Each entry in the hash table, a xw_list pointer, is
//                   a linked list of these ID-pointer associations. The hash
//                   function is a simple modulo.
//
//  RELATIONS      : xw_list
//  SEE ALSO       : xapplication
//  LAST MODIFIED  : Feb 08, 1998
///////////////////////////////////////////////////////////////////////////
*/

class xw_hash
{
   xw_list    **table;   //  The actual hash table.
   unsigned   tabsize;   //  Nr. of entries in the hash table.

   unsigned   Hash(XID win)
   {
      return win % tabsize;
   }

public:

   xw_hash()
   {
      tabsize = 100;

      table = new xw_list * [tabsize];
      for (unsigned i=0; i < tabsize; i++)
      {
         table[i] = 0;
      }
   }

   xw_hash(unsigned size)
   {
      tabsize = size;

      table = new xw_list * [size];
      for (unsigned i=0; i < tabsize; i++)
      {
         table[i] = 0;
      }
   }

   ~xw_hash()
   {
      delete[] table;
   }

   //   Add a new entry in the hash table.

   void xh_add(XID win_id, window *win_ptr)
   {
      unsigned   h;   // The hash value;

      h = Hash(win_id);
      table[h] = new xw_list(win_id, win_ptr, table[h]);
   }

   //  Delete an entry from the hash table.

   void xh_del(XID win_id)
   {
      unsigned   h;         // The hash value;
      xw_list    *entry;    // The one we want to remove.
      xw_list    *previous; // The one just before entry.

      h = Hash(win_id);

      //   find the window's entry in the list.
      previous = 0;
      entry = table[h];
      while (entry && entry->id != win_id)
      {
         previous = entry;
         entry = entry->next;
      }

      if (previous)
      {
         //   Re-link the list, thus excluding entry.
         previous->next = entry->next;
      }
      else
      {
         //  It was the first entry.
         table[h] = entry->next;
      }
      delete entry;
   }

   window * operator[](XID win_id)
   {
      unsigned   h;   // The hash value;
      xw_list    *entry;

      h = Hash(win_id);
      for (entry = table[h]; entry && entry->id != win_id; entry = entry->next);
      if (entry)
      {
         return entry->win;
      }
      else
      {
         return 0;
      }
   }
};


/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : xapplication
//  BASECLASS      :
//  MEMBERS        : wintab   : The table of maintained windows.
//  OPERATORS      :
//  METHODS        : Initialize
//                   ParseArguments
//                   SetupResources
//                   UserInit
//                   DispatchEvent
//                   Cleanup
//                   add_window
//                   remove_window
//
//  DESCRIPTION    : Each X Windows application MUST have one object of a
//                   class derived from xapplication.
//                   The xapplication class implements the main thread of
//                   control: initialization, event-loop and termination.
//                   It maintains a list of all windows that are created by
//                   the application.
//
//  RELATIONS      : xw_hash
//  SEE ALSO       :
//  LAST MODIFIED  : Feb 23, 1998
///////////////////////////////////////////////////////////////////////////
*/

class xapplication
{
   xw_hash   wintab;

   window    *Pgrab_window;

   managed_window    *Top_window;

public:

   xapplication();

   virtual ~xapplication() {}

   void add_window(XID w_id, window *w_ptr);

   void remove_window(XID w_id);

   void Pgrab(window *w_ptr);
   void Topwin(managed_window *w_ptr);

   virtual void Initialize(void);

   virtual void ParseArguments(int argc, char *argv[]);

   virtual void SetupResources(void);

   virtual void UserInit(void);

   virtual int DoCommand(int code);

   virtual void Cleanup(void);

   int  DispatchEvent(XEvent ev);
   int  SendCommand(int code);

};

extern xapplication *XApp;

extern color Black, White;
extern color menu_normal, menu_highlight;
extern color inside_3D, light_rim_3D, dark_rim_3D;

extern font  default_font;
extern font  fixed;

extern stipple dim_pattern;

extern cursor  hand_cursor;
extern cursor  text_cursor;
extern cursor  busy_cursor;

extern gc    menu_normal_gc;
extern gc    menu_dimmed_gc;
extern gc    inside_3D_gc, light_3D_gc, dark_3D_gc;
extern gc    red_gc, green_gc;
extern gc    white_gc;
extern gc    text_normal_gc;
extern gc    edit_cursor_gc;

extern pixmap nopicture;

#endif  /*  XAPPL_H */
