/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : xwindow.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Definition of the window class
**
**  EXPORTED OBJECTS : class window
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Jul 28, 2000
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: xwindow.h,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: xwindow.h,v 1.1 2002/07/25 08:01:27 arjen Exp $"; */

#ifndef AXE_XWINDOW_H
#define AXE_XWINDOW_H

#include <vector>
#include <X11/Xlib.h>
#include "geometry.h"
#include "gc.h"
#include "pixmap.h"
#include "String.h"

#define GrabPointerMask    (1L << 25)

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : window
//  BASECLASS      :
//  MEMBERS        : win_id     : X server id of the window
//                   children   : Linked list of child windows.
//                   parent     : The parent window (0 if main window)
//                   selection_mask : Mask of selected events.
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    : A window object encapsulates a window on the X workstation.
//                   A window is a rectangular area on the screen, used
//                   for graphical output. Input from mouse and keyboard
//                   is handled by event-handling functions in a window.
//
//  RELATIONS      :
//  SEE ALSO       : xapplication
//  LAST MODIFIED  : Feb 13, 1998
///////////////////////////////////////////////////////////////////////////
*/

class window;

   struct win_list
   {
      window   *child;
      win_list *next;
   };


   struct win_message
   {
      int     msg_id;
      window *from;
   };

class window
{
   XID   win_id;

   unsigned   w, h;           //  Size of the window

   long       selection_mask;

   void add_child(window *ch);
   void remove_child(window *ch);

protected:

   window     *parent;

public:

   win_list   *children;
   XID ID()
   {
      return win_id;
   }

   //  Event handlers
   
   virtual int EV_Expose(XExposeEvent);
   virtual int EV_NoExpose(XNoExposeEvent);
   virtual int EV_KeyPress(XKeyEvent);
   virtual int EV_KeyRelease(XKeyEvent);
   virtual int EV_ButtonPress(XButtonEvent);
   virtual int EV_ButtonRelease(XButtonEvent);
   virtual int EV_MotionNotify(XMotionEvent);
   virtual int EV_EnterNotify(XCrossingEvent);
   virtual int EV_LeaveNotify(XCrossingEvent);
   virtual int EV_FocusIn(XFocusChangeEvent);
   virtual int EV_FocusOut(XFocusChangeEvent);
   virtual int EV_ConfigureNotify(XConfigureEvent);
   virtual int EV_ClientMessage(XClientMessageEvent);

   virtual int ParentResized(int w, int h);

   virtual void ChildMessage(win_message &msg);

   void    ParentMessage(int id);

   int EV_Default(XEvent);

   window(int x=0, int y=0, unsigned w=320, unsigned h=256, int bw = 4);
   window(window &, int x=0, int y=0,
          unsigned int w=100, unsigned int h=100, int bw = 1);
   virtual ~window();

   //   Mapping and unmapping

   void Map(void)
   {
      XMapWindow(stddpy.Dpy(), win_id);
   }
   
   void Realize(void)
   {
      XMapSubwindows(stddpy, win_id);
      XMapRaised(stddpy, win_id);
   }

   void Unmap(void)
   {
      XUnmapWindow(stddpy.Dpy(), win_id);
   }
   
   //  Size and position

   void Size(unsigned &width, unsigned &height)
   {
      width = w;
      height = h;
   }

   size Size(void)
   {
      return size(w, h);
   }

   void Resize(unsigned width, unsigned height);
   void Resize(size sz);
   void Move(int x, int y);

   //   Border and background

   void Border(unsigned int borderwidth)
   {
      XSetWindowBorderWidth(stddpy, win_id, borderwidth);
   }

   void Background(unsigned long pixel);
   void WindowGravity(int gravity);
   void Clear(void);

   long SelectInput(long events, int on);
   long SelectInput(void)
   {
      return selection_mask;
   }

   void SetFocus(void)
   {
      XSetInputFocus(stddpy, win_id, RevertToPointerRoot, CurrentTime);
   }

   int  GrabPointer(Cursor curs);
   int  UngrabPointer(void);
   void DefineCursor(Cursor curs)
   {
      XDefineCursor(stddpy, ID(), curs);
   }
   void UndefineCursor(void)
   {
      XUndefineCursor(stddpy, ID());
   }

   //   Graphic drawing functions
   
   void DrawString(const gc &graphic, int x, int y, String str) const
   {
      XDrawString(stddpy, win_id, graphic, x, y, str, ~str);
   }

   void DrawString(const gc &graphic, point p, String str) const
   {
      XDrawString(stddpy, win_id, graphic, p.x, p.y, str, ~str);
   }

   void DrawLine(const gc &graphic, int x1, int y1, int x2, int y2) const
   {
      XDrawLine(stddpy, win_id, graphic, x1, y1, x2, y2);
   }

   void DrawLine(const gc &graphic, point p1, point p2) const
   {
      XDrawLine(stddpy, win_id, graphic, p1.x, p1.y, p2.x, p2.y);
   }

   void DrawLine(const gc &graphic, std::vector<point> &l, int mode = CoordModeOrigin) const
   {
      int npoints  = l.size();
      XPoint  *points = new XPoint[npoints];

      for (int i = 0; i < npoints; i++)
      {
         points[i].x = l[i].x;
         points[i].y = l[i].y;
      }

      XDrawLines(stddpy, win_id, graphic, points, npoints, mode);
   }

   void FillPolygon(const gc &graphic, std::vector<point> &l,
                    int mode = CoordModeOrigin, int shape = Complex) const
   {
      int npoints  = l.size();
      XPoint  *points = new XPoint[npoints];

      for (int i = 0; i < npoints; i++)
      {
         points[i].x = l[i].x;
         points[i].y = l[i].y;
      }

      XFillPolygon(stddpy, win_id, graphic, points, npoints, shape, mode);
   }

   void DrawRectangle(const gc &graphic, point p, size s) const
   {
      XDrawRectangle(stddpy, win_id, graphic, p.x, p.y, s.w, s.h);
   }

   void DrawRectangle(const gc &graphic, int x, int y, unsigned w, unsigned h) const
   {
      XDrawRectangle(stddpy, win_id, graphic, x, y, w, h);
   }

   void DrawRectangle(const gc &graphic, rectangle r) const
   {
      DrawRectangle(graphic, r.Origin(), r.Size());
   }

   void DrawRectangle(const gc &graphic, point p1, point p2) const
   {
      DrawRectangle(graphic, rectangle(p1, p2));
   }

   void FillRectangle(const gc &graphic, point p, size s) const
   {
      XFillRectangle(stddpy, win_id, graphic, p.x, p.y, s.w, s.h);
   }

   void FillRectangle(const gc &graphic, int x, int y, unsigned w, unsigned h) const
   {
      XFillRectangle(stddpy, win_id, graphic, x, y, w, h);
   }

   void FillRectangle(const gc &graphic, rectangle r) const
   {
      FillRectangle(graphic, r.Origin(), r.Size());
   }

   void FillRectangle(const gc &graphic, point p1, point p2) const
   {
      FillRectangle(graphic, rectangle(p1, p2));
   }

   void DrawArc(const gc &graphic, int x, int y, unsigned w, unsigned h,
                int angle1, int angle2) const
   {
      XDrawArc(stddpy, win_id, graphic, x, y, w, h, angle1, angle2);
   }

   void FillArc(const gc &graphic, int x, int y, unsigned w, unsigned h,
                int angle1, int angle2) const
   {
      XFillArc(stddpy, win_id, graphic, x, y, w, h, angle1, angle2);
   }

   void DrawPixmap(const gc &graphic, int x, int y, const pixmap &pm) const
   {
      XCopyArea(stddpy, pm, win_id, graphic, 0, 0,
                pm.Size().w, pm.Size().h, x, y);
   }
};

class managed_window : public window
{
   unsigned long bg;
   int      WM_Delete_Command;  //  Command to send op 'close' button.

public:

   managed_window()
   {
      WM_Delete_Command = 0;
      Atom    wm_delete;

      wm_delete = XInternAtom(stddpy, "WM_DELETE_WINDOW", true);
      bg = 0;

      XStoreName(stddpy, ID(), "AXE Test");

      XSetWMProtocols(stddpy, ID(),&wm_delete , 1);

      SelectInput(StructureNotifyMask, 1);
      SelectInput(FocusChangeMask, 1);

   }

   managed_window(char *name)
   {
      WM_Delete_Command = 0;
      Atom    wm_delete;

      wm_delete = XInternAtom(stddpy, "WM_DELETE_WINDOW", true);
      bg = 0;

      XStoreName(stddpy, ID(), name);

      XSetWMProtocols(stddpy, ID(),&wm_delete , 1);

      SelectInput(StructureNotifyMask, 1);
      SelectInput(FocusChangeMask, 1);

   }

   void Title(char *name)
   {
      XStoreName(stddpy, ID(), name);
   }

   void Command_WhenClosed(int cmd)
   {
      WM_Delete_Command = cmd;
   }

   virtual int EV_FocusIn(XFocusChangeEvent);
   virtual int EV_ClientMessage(XClientMessageEvent ev);

   virtual int DoCommand(int code)
   {
      return -1;
   }
};

/*
 *  Predefined message id's in a win_message
 */

#define EDIT_ENTERED       1
#define EDIT_FOCUSLOST     2
#define SCROLL_POSITIONED  3
#define TABLE_ROW_SELECT   4

#endif /* AXE_XWINDOW_H */

